#ifndef demucc_h
#define demucc_h

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
typedef char           int8_t;
typedef short          int16_t;
typedef int            int32_t;
typedef unsigned char  uint8_t;
typedef unsigned short uint16_t;
typedef unsigned int   uint32_t;

#define TPQN 24
#define TIMEBASE (TPQN * 4)
#define STACK_SIZE 32 // muslib  24
#define DRUMNAME_MAX 16
#define MMLBUF_SIZE (1024 * 1024)
#define MML_COLUMNS 78

#define TRATTR_DRUMS 1
#define TRATTR_NOOFF 2

struct mmlbuf {
    char* mml;
    char  buf[MMLBUF_SIZE];
};

struct pmd {
    /* command line params */
    int mml_columns;
    int use_ticks; // instead of beats

    /* global context */
    uint8_t* buffer;
    long buffer_size;
    uint8_t* p;
    uint8_t** parts;
    uint8_t* drums;
    int num_parts;
    char* title;
    char* title2;

    /* current part context */
    int part;
    int stack[STACK_SIZE];
    int sp;
    int len;
    int track_attr;
    int legato;
    int oct;
    int porsw;
    int porpd;
    int porlen;
    int return_addr;
    int tick;

    struct mmlbuf* mmlbuf;
    int column;
    int newline;
};

/* part.c */
int extract_drums(struct pmd* pmd);
int read_notes(struct pmd* pmd, uint8_t** pp);

/* util.c */
uint16_t read_u16(uint8_t** pp);
uint8_t read_u8(uint8_t** pp);
int tick2beat(struct pmd* pmd, int tick, char* notes);
void get_note(struct pmd* pmd, int note, int tick, int por_pre);
int get_drumname(int note, char* buffer, size_t size);
void reset_part_ctx(struct pmd* pmd);
int mml_printf(struct pmd* pmd, char* format, ...);
int mml_vprintf(struct pmd* pmd, char* format, va_list vp);
int mml_newline(struct pmd* pmd);
int crlf2semicolon(char* string, char** outbuf);

/* commands.c */
int read_commands(struct pmd* pmd, uint8_t n, uint8_t** pp);

/* mmlbuf.c */
struct mmlbuf* mmlbuf_new();
void mmlbuf_free(struct mmlbuf* mmlbuf);
void mmlbuf_reset(struct mmlbuf* mmlbuf);
char* mmlbuf_buf(struct mmlbuf* mmlbuf);
int mmlbuf_append(struct mmlbuf* mmlbuf, char* format, ...);
int mmlbuf_appendv(struct mmlbuf* mmlbuf, char* format, va_list ap);

#endif // !demucc_h
